//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#include <vcl.h>
#include <windows.h>
#pragma hdrstop
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#include "f_ConditionData.h"
#include "f_ConditionConfig.h"
#include "f_ConditionLogic.h"
#include "f_VD15xProxy.h"

#ifndef _DEBUG
#include <stdio.h>
#endif

#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
  return 1;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void CondCheckDeleted(CFD*& mfd, const FilterActivation *fa, const FilterFunctions *ff)
{
  if (cond_dd.CheckDeleted(mfd)) {
    MessageBox(0,
    "Conditional filter data lost!\n"
    "This can occur if you delete conditional filter in the 'filters' dialog\n"
    "and then leave this dialog with cancel!\n\n"
    "Conditional data are recreated to avoid crash\n"
    "but the settings are lost, sorry!",
    "Conditional filter error", MB_ICONWARNING | MB_OK);

    mfd = new CFD();

    mfd->OwnFilter     = fa->filter;
    mfd->OwnFilterData = fa->filter_data;
    mfd->ff            = ff;
  }
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CondInitProc(FilterActivation *fa, const FilterFunctions *ff)
{
dout(__FUNC__);
  CFD* mfd = new CFD();
  *reinterpret_cast<CFD**>(fa->filter_data) = mfd;

  mfd->OwnFilter     = fa->filter;
  mfd->OwnFilterData = fa->filter_data;
  mfd->ff            = ff;

  int Result = 0;

  try {
    // VD 1.5.x attach to filter
    if (mfd->Filter) {
      CFilterDefinitionHelper fdh(fd_conditional);
      fdh.Attach(mfd->Filter);
    }

    if (mfd->Filter && mfd->Filter->initProc && mfd->Branches) {
dout(__FUNC__ " should never see it ******");
      CBranch* Branch = mfd->Branches;
      while (Branch) {
        // fake
        fa->filter      = mfd->Filter;
        fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call initProc (%#x)", fa->filter_data);
        if (mfd->Filter->initProc(fa, ff))
          ++Result;

        Branch = Branch->Next;
      }
    }
  } __finally {
    // restore data
    fa->filter      = mfd->OwnFilter;
    fa->filter_data = mfd->OwnFilterData;
  }

  return Result;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void CondDeinitProc(FilterActivation *fa, const FilterFunctions *ff)
{
dout(__FUNC__);
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  if (mfd->Filter && mfd->Filter->deinitProc && mfd->Branches) {
    CBranch* Branch = mfd->Branches;

    mfd->Branches->CallEndProc(fa, ff);
    while (Branch) {
      // fake
      fa->filter      = mfd->Filter;
      fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call deinitProc (%#x)", fa->filter_data);

      mfd->Filter->deinitProc(fa, ff);
      Branch->pInstance->ForceNoDeinit();

      Branch = Branch->Next;
    }
  }

  // restore data
  fa->filter      = mfd->OwnFilter;
  fa->filter_data = mfd->OwnFilterData;

  if (mfd->Branches) {
dout(__FUNC__ " deleting branches");
    mfd->Branches->DeleteBranch(fa, ff);
    mfd->Branches = 0;
  }

  mfd->PreviewBranch = 0;

  // VD 1.5.x detach from filter
  if (mfd->Filter) {
    CFilterDefinitionHelper fdh(fd_conditional);
    fdh.Detach(mfd->Filter);

    mfd->Filter = 0;
  }

  delete mfd;
  cond_dd.AddDeleted(mfd);

  *reinterpret_cast<CFD**>(fa->filter_data) = 0;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void DoNothing(FilterActivation* fa, bool SwapBuffer)
{
dout(__FUNC__);
  if (SwapBuffer) {
dout(__FUNC__ " swapping");
    PixDim w = fa->dst.w, h = fa->dst.h;
    Pixel32 *src = fa->src.data, *dst = fa->dst.data;
    do {
      memcpy(dst, src, 4 * w);
      src = (Pixel32 *)((char *)src + fa->src.pitch);
      dst = (Pixel32 *)((char *)dst + fa->dst.pitch);
    } while(--h);
  }
}

int  CondRunProc(const FilterActivation *cfa, const FilterFunctions *ff)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(cfa->filter_data), cfa, ff);
dout(__FUNC__);

  FilterActivation* fa = const_cast<FilterActivation*>(cfa);
  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  if (mfd->Disabled) {
/*
//   we can throw an exception, but do we really want to do it?
//   it seems that after that nothing works, therefore do nothing and say ok
    try {
      if (*reinterpret_cast<char*>(ff->Except) != 0x55) // != push ebp
        return -1;
    } __except(1) {
      return -1;
    }
    ff->Except("Conditional disabled: %s", mfd->DisabledReason);
*/
    DoNothing(fa, mfd->HaveToSwap);
    return 0;
  }

  if (!mfd->Filter || !mfd->Branches) {
    CNumeric* PreviewMonitor = mfd->MonitorPreview ? *mfd->MonitorPreview : 0;
    if (PreviewMonitor) {
      CExecutionContext Context(fa, -1);
      int Index = 0;
      do {
        try {
          try {
            long Result = PreviewMonitor->GetValue(Context);
            PostMessage(mfd->MonitorPreviewHandle, WM_USER + 11, Index, Result);
          } catch (...) {}
        } __except(1) {}
        ++Index;
        PreviewMonitor = reinterpret_cast<CNumeric*>(PreviewMonitor->Next);
      } while(PreviewMonitor);
      PostMessage(mfd->MonitorPreviewHandle, WM_USER + 12, 0, 0);
    }

    DoNothing(fa, mfd->HaveToSwap);
    return 0;
  }

  int   Counter = 0;
  int   Result  = 0;

  CCondFilterInstance* Data;
  CExecutionContext Context(fa, mfd->Branches->NeedFlags);
  {
    CNumeric* PreviewMonitor = mfd->MonitorPreview ? *mfd->MonitorPreview : 0;
    if (PreviewMonitor) {
      Context.ForceNeedAnything();
      int Index = 0;
      do {
        try {
          try {
            long Result = PreviewMonitor->GetValue(Context);
            PostMessage(mfd->MonitorPreviewHandle, WM_USER + 11, Index, Result);
          } catch (...) {}
        } __except(1) {}
        ++Index;
        PreviewMonitor = reinterpret_cast<CNumeric*>(PreviewMonitor->Next);
      } while(PreviewMonitor);
      PostMessage(mfd->MonitorPreviewHandle, WM_USER + 12, 0, 0);
    }
  }

  if (mfd->PreviewBranch) {
    Data = mfd->PreviewBranch->pInstance;
    if (mfd->MonitorPreviewHandle)
      PostMessage(mfd->MonitorPreviewHandle, WM_USER + 13, 0, -2);
  } else {
    Data = mfd->Branches->Evaluate(Context, Counter);
    if (mfd->MonitorPreviewHandle)
      PostMessage(mfd->MonitorPreviewHandle, WM_USER + 13, 0, Counter);
  }

  if (Data) {
    if (! Data->Started)
      Data->CallStartProc(fa, ff);

    // fake
    fa->filter      = mfd->Filter;
    fa->filter_data = Data->pData;

dout(__FUNC__ " time to play (%#x)", fa->filter_data);

    try {
      Result = mfd->Filter->runProc(fa, ff);
    } __finally {
      // restore data
      fa->filter      = mfd->OwnFilter;
      fa->filter_data = mfd->OwnFilterData;
    }
  } else {
    DoNothing(fa, mfd->HaveToSwap);
  }
  return Result;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
long CondParamProc(FilterActivation *fa, const FilterFunctions *ff)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  mfd->Disabled = false;
  mfd->DisabledReason = 0;

  if (! mfd->Filter) {
dout(__FUNC__ " no filter");
    mfd->HaveToSwap = false;
    return 0;
  }

  if (! mfd->Filter->paramProc) {
dout(__FUNC__ " no paramProc, default handling");
    mfd->HaveToSwap = true;
    return FILTERPARAM_SWAP_BUFFERS;
  }

  long Flags = 0;
  if (mfd->Branches) {
    CBranch* Branch = mfd->Branches;

    // save the original settings
    VFBitmapInfo Original(fa->dst);
dout(__FUNC__ " original  settings: %dx%d(%d)", Original.w, Original.h, Original.offset);

    // fake
    fa->filter      = mfd->Filter;
    fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call paramProc (%#x)", fa->filter_data);

    // ask new settings
    long NewFlags = mfd->Filter->paramProc(fa, ff);
    VFBitmapInfo NewSettings(fa->dst);
    bool Valid = true;
dout(__FUNC__ " requested settings: %dx%d(%d)%s%s%s", NewSettings.w, NewSettings.h, NewSettings.offset,
NewFlags & FILTERPARAM_SWAP_BUFFERS ? " swap" : "", NewFlags & FILTERPARAM_NEEDS_LAST ? " last" : "", NewFlags > 10 ? " lag" : "");

    Branch = Branch->Next;
    while (Branch) {
      // fake
      fa->filter      = mfd->Filter;
      fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call paramProc (%#x)", fa->filter_data);
      // restore dst
      Original.Restore(fa->dst);

      long TempFlags = mfd->Filter->paramProc(fa, ff);
      if (NewFlags != TempFlags) {
dout(__FUNC__ " fails due to     : %s%s%s",
TempFlags & FILTERPARAM_SWAP_BUFFERS ? " swap" : "", TempFlags & FILTERPARAM_NEEDS_LAST ? " last" : "", TempFlags > 10 ? " lag" : "");
        // uuups
        Valid = false;
        mfd->Disabled = true;
        mfd->DisabledReason = "different flags in branches";
        break;
      }

      VFBitmapInfo Temp(fa->dst);
      if (NewSettings != Temp) {
dout(__FUNC__ " fails due to     : %dx%d(%d)", Temp.w, Temp.h, Temp.offset);
        // uuups
        Valid = false;
        mfd->Disabled = true;
        mfd->DisabledReason = "different settings in branches";
        break;
      }

      Branch = Branch->Next;
    }
    if (Valid && (! mfd->Branches->HasElse()) && (Original != NewSettings)) {
dout(__FUNC__ " fails due to     : else required");
      Valid = false;
      mfd->Disabled = true;
      mfd->DisabledReason = "else required";
    }
    if (Valid) {
      Flags = NewFlags;
    } else {
      Original.Restore(fa->dst);
    }
  }

  mfd->HaveToSwap = Flags & FILTERPARAM_SWAP_BUFFERS;

  // restore data
  fa->filter      = mfd->OwnFilter;
  fa->filter_data = mfd->OwnFilterData;

  return Flags;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CondConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hWnd)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;
  CFD  mfdold = *mfd;

  mfdold.PreviewBranch = 0;
  mfd->PreviewBranch = 0;
  if (mfd->Branches)
    mfd->Branches = mfd->Branches->Clone();

  bool FilterWasEmpty = mfd->Filter == 0;

  Application->Handle = hWnd;
//  Application->Handle = GetParent(hWnd);
  TConditionConfig* Form = new TConditionConfig(Application, fa, ff);
  int Result = Form->ShowModal();

  mfd->PreviewBranch = 0;
  if (Result == mrOk) {
    if (mfdold.Branches) {
      mfdold.Branches->DeleteBranch(fa, ff);
      mfdold.Branches = 0;
    }
  } else {

    // it is possible to attach in the configuration window
    // VD 1.5.x detach from filter
    if (FilterWasEmpty && mfd->Filter) {
      CFilterDefinitionHelper fdh(fd_conditional);
      fdh.Detach(mfd->Filter);
    }

    if (mfd->Branches)
      mfd->Branches->DeleteBranch(fa, ff);
    *mfd = mfdold;
    mfdold.Branches = 0;
  }

  delete Form;
  return Result != mrOk; // 0 on Ok, something else on Cancel
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void CondStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *buf)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);

  if (mfd->Disabled)
    sprintf(buf, " DISABLED: %s", mfd->DisabledReason);
  else if (!mfd->Filter)
    strcpy(buf, " NOT INITIALIZED");
  else {
    CBranch* Branch = mfd->Branches;
    int Counter = 0;
    while (Branch) {++Counter; Branch = Branch->Next;}
    sprintf(buf, " apply '%s' - %d branches", mfd->Filter->name, Counter);
  }
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CondStartProc(FilterActivation *fa, const FilterFunctions *ff)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  int Result = 0;

  try {
    if (mfd->Filter && mfd->Filter->startProc && mfd->Branches) {
      CBranch* Branch = mfd->Branches;
      while (Branch) {
        // fake
        fa->filter      = mfd->Filter;
        fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call startProc (%#x)", fa->filter_data);

        if (mfd->Filter->startProc(fa, ff))
          ++Result;

        Branch->pInstance->Started = true;
        Branch = Branch->Next;
      }
    }
  } __finally {
    // restore data
    fa->filter      = mfd->OwnFilter;
    fa->filter_data = mfd->OwnFilterData;
  }

  return Result;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CondEndProc(FilterActivation *fa, const FilterFunctions *ff)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  int Result = 0;

  if (mfd->Filter && mfd->Filter->endProc && mfd->Branches) {
    CBranch* Branch = mfd->Branches;
    while (Branch) {
      // fake
      fa->filter      = mfd->Filter;
      fa->filter_data = Branch->pInstance->pData;

dout(__FUNC__ " call endProc (%#x)", fa->filter_data);

      if (mfd->Filter->endProc(fa, ff))
        ++Result;

      Branch->pInstance->Started = false;
      Branch = Branch->Next;
    }
  }

  // restore data
  fa->filter      = mfd->OwnFilter;
  fa->filter_data = mfd->OwnFilterData;

  return Result;
}

#include <ScriptInterpreter.h>
#include <ScriptValue.h>
extern "C" CFormula* parseFormula(const char*);
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void CondScriptConfigCondition(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
dout(__FUNC__);

  FilterActivation* fa = reinterpret_cast<FilterActivation*>(lpVoid);
  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  if (!mfd->Filter) {
    // should never happen
    return;
  }

  if (argc < 1) return;
  if (!argv[0].isString()) return;

  int  offset = 0;
  bool ElsePossible = true;
  if (mfd->Branches && mfd->Branches->HasElse())
    ElsePossible = false;

  const char* formulaText = *argv[offset].asString();
  CBranch* Branch = 0;
  if (strlen(formulaText)) {
    CFormula* formula = parseFormula(formulaText);
    if (formula)
      Branch = new CIFBranch(fa, mfd->ff, formula, formulaText);
  } else if (ElsePossible) {
    Branch = new CELSEBranch(fa, mfd->ff);
  }

  if (!Branch) return;
  if (mfd->Branches) {
    CBranch *LastBranch = mfd->Branches;
    while (LastBranch->Next) LastBranch = LastBranch->Next;
    LastBranch->Next = Branch;
  } else {
    mfd->Branches = Branch;
  }

  if ((argc > 1) && (mfd->Filter->script_obj) && (mfd->Filter->script_obj->func_list[0].func_ptr))
  {
    // now we have to config the instance
    // fake
    fa->filter      = mfd->Filter;
    fa->filter_data = Branch->pInstance->pData;

    ((ScriptVoidFunctionPtr)
      mfd->Filter->script_obj->func_list[0].func_ptr)  (isi, lpVoid, argv + 1, argc - 1);

    // restore data
    fa->filter      = mfd->OwnFilter;
    fa->filter_data = mfd->OwnFilterData;
  }
}
void CondScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
dout(__FUNC__);
  FilterActivation* fa = reinterpret_cast<FilterActivation*>(lpVoid);
  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;

  if (mfd->Filter) {
    // should never happen
    return;
  }

  const char* FilterName = *argv[0].asString();

  #pragma warn -8004 // never used
  {
    int BranchesCount = argv[1].asInt(); // is not used currently
  }
  #pragma warn .8004

  {
    CFilterDefinitionHelper fdh(mfd->OwnFilter);
    FilterDefinition* Filter = fdh.FindFilter(FilterName, mfd->OwnFilter);

    if (!Filter) return; // nothing to do

    fdh.Attach(Filter);

    mfd->Filter = Filter;
  }
}

ScriptFunctionDef condScriptFun_tbl[] = {
  {(ScriptFunctionPtr) CondScriptConfig, "Config", "0si"},
  {(ScriptFunctionPtr) CondScriptConfig, 0, "0s."},
  {(ScriptFunctionPtr) CondScriptConfigCondition, "Condition", "0s"},
  {(ScriptFunctionPtr) CondScriptConfigCondition, 0, "0s."},
  {0}
};

CScriptObject condScriptObj = {0, condScriptFun_tbl};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void handleFssData(char* buf)
{
  char* pParPos = strchr(buf, '(');
  if (!pParPos) return;

  memmove(buf, pParPos, strlen(pParPos) + 1);
  buf[0] = ',';
}
void handleEscSeq(char* dst, int dstsize, const char* src)
{
  while(*src && --dstsize) {
    switch (*src) {
      case '"':  *dst++ = '\\'; *dst++ = *src++; break;
      case '\\': *dst++ = '\\'; *dst++ = *src++; break;
      case '\t': *dst++ = '\\'; *dst++ = 't'; src++; break;
      case '\r': *dst++ = '\\'; *dst++ = 'r'; src++; break;
      case '\n': *dst++ = '\\'; *dst++ = 'n'; src++; break;
      default: *dst++ = *src++;
    }
  }
  *dst = 0;
}

// VirtualDub uses structures like these to embed the FilterActivation
// We try to reconstruct the original structure to be able to find
// the index of our structure in the filter chain
class VDListNode {
  public:
    VDListNode *next, *prev;
};
class VDFilterInstance : public VDListNode, public FilterActivation {
  public:
    int Something;
};
int GetInstanceIndex(FilterActivation *fa)
{
  int Result = -2;

  bool AllRight = true;
  try {
    VDFilterInstance* Inst     = static_cast<VDFilterInstance*>(fa);
    VDFilterInstance* InstNext = (VDFilterInstance*) static_cast<VDFilterInstance*>(fa)->next;
    VDFilterInstance* InstPrev = (VDFilterInstance*) static_cast<VDFilterInstance*>(fa)->prev;
    if (InstNext)
      AllRight = AllRight && (InstNext->prev == Inst);
    if (InstPrev)
      AllRight = AllRight && (InstPrev->next == Inst);
  } __except(1) {
    // Hmm, the structur has changed in the last time
    dout("Conditional: Hmm, the structur has changed in the last time");

    AllRight = false;
  }
  if (AllRight) {
    VDFilterInstance* Inst = static_cast<VDFilterInstance*>(fa);
    while (Inst) {
      Result++;
      Inst = (VDFilterInstance*) Inst->prev;
    }
  }
  return Result;
}
// Due to the matter how conditional saves the data, we have no chance
// to allow the nesting of conditional
bool CondScriptStrProc(FilterActivation *fa, const FilterFunctions *ff, char *buffer, int cnt)
{
CondCheckDeleted(*reinterpret_cast<CFD**>(fa->filter_data), fa, ff);
dout(__FUNC__ " we have %d chars", cnt);

  CFD* mfd = *reinterpret_cast<CFD**>(fa->filter_data);
  mfd->OwnFilterData = fa->filter_data;
  if (!mfd->Filter) return false;

  int iFilter = GetInstanceIndex(fa);

  // count branches
  int branchesCnt = 0;
  CBranch* Branch = mfd->Branches;
  while(Branch)
  {
    ++branchesCnt;
    Branch = Branch->Next;
  }

  char tempBuffer[2048];
  char filterBuffer[2048];
  handleEscSeq(tempBuffer, sizeof(tempBuffer), mfd->Filter->name);
  _snprintf(buffer, cnt, "Config(\"%s\",%d)", tempBuffer, branchesCnt);
  cnt    -= strlen(buffer);
  buffer += strlen(buffer);

  Branch = mfd->Branches;
  while (Branch)
  {
    if (Branch->IsElse)
      tempBuffer[0]=0;
    else
      handleEscSeq(tempBuffer, sizeof(tempBuffer), static_cast<CIFBranch*>(Branch)->pFormula->GetFormulaText());

    if (mfd->Filter->fssProc) {
      // fake
      fa->filter      = mfd->Filter;
      fa->filter_data = Branch->pInstance->pData;
      if (!mfd->Filter->fssProc(fa, ff, filterBuffer, sizeof(filterBuffer)))
        strcpy(filterBuffer, ")");
      else
        handleFssData(filterBuffer);
    } else {
      strcpy(filterBuffer, ")");
    }

    int cnt_required = strlen(tempBuffer) + strlen(filterBuffer) + 50;
    if (cnt > cnt_required) {
      // , and ) are in filterBuffer
      _snprintf(buffer, cnt,
        ";\n"
        "VirtualDub.video.filters.instance[%d].Condition"
        "(\"%s\"%s", iFilter, tempBuffer, filterBuffer);
      cnt    -= strlen(buffer);
      buffer += strlen(buffer);
    } else {
      // not enought space, break and ignore the rest
dout(__FUNC__ " not enought place to store all the data");
      break;
    }

    Branch = Branch->Next;
  }

  // restore data
  fa->filter      = mfd->OwnFilter;
  fa->filter_data = mfd->OwnFilterData;

  return true;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
ExternalFunction GetExternalFunction(const char* Name)
{
  for (int i = 0; i < 100; ++i) {
    if (! ExternalFunctions[i].name) break;

    if (!stricmp(ExternalFunctions[i].name, Name))
      return ExternalFunctions[i].fun;
  }
  return 0;
}
bool installPlugin(const char* Name, int& Index)
{
 HINSTANCE hInstance = LoadLibrary(Name);
dout("loading plugin %s....", Name);
  if (!hInstance)  return false;

  TypeGetExtFunctions fun = (TypeGetExtFunctions) GetProcAddress(hInstance, "GetExtFunctions");
  if (!fun)
    fun = (TypeGetExtFunctions) GetProcAddress(hInstance, "_GetExtFunctions");
  if (!fun)
    return false;

  ExternalFunctionInfo* Info = 0;
  fun(&Info);
  while (Info && Info->name && (Index < 100)) {
dout("loaded %s", Info->name);
    ExternalFunctions[Index] = *Info;
    ExternalFunctions[Index].hInstance = hInstance;
    ++Info;
    ++Index;
  }
  return true;
}
void installExternalPlugins()
{
  ZeroMemory(ExternalFunctions, sizeof(ExternalFunctions));
  char szFile[MAX_PATH], szFile2[MAX_PATH];
  char *lpszName;

  GetModuleFileName(NULL, szFile, sizeof(szFile));
  GetFullPathName(szFile, sizeof(szFile2), szFile2, &lpszName);
  strcpy(lpszName, "plugins\\*.vdc");
  lpszName = strchr(lpszName, '\\') + 1;
  HANDLE h;
  WIN32_FIND_DATA wfd;

  h = FindFirstFile(szFile2, &wfd);
  int Counter = 0;

  if (h != INVALID_HANDLE_VALUE) {
    do {
      try {
        strcpy(lpszName, wfd.cFileName);
        installPlugin(szFile2, Counter);
      } __except(1) {}
    } while(FindNextFile(h, &wfd));
  }
  FindClose(h);
}
void RemoveExternalPlugins()
{
  HINSTANCE hLast = (HINSTANCE) -1;

  for (int i = 0; i < 100; ++i) {
    HINSTANCE hCurrent = ExternalFunctions[i].hInstance;
    if (!hCurrent) break;
    if (hCurrent != hLast) {
      FreeLibrary(hCurrent);
      hLast = hCurrent;
    }
  }
  ZeroMemory(ExternalFunctions, sizeof(ExternalFunctions));
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
FilterDefinition filterDef_conditional = {
  0, 0, 0,             // next, prev, module
  "conditional",       // name
  "Allows you to apply other filter with settings depend on some condition.",
                       // desc
  "Dmitri Schamschurko",
                       // maker

  0,                   // private_data
  sizeof(CFD*),        // inst_data_size

  CondInitProc,        // initProc
  CondDeinitProc,      // deinitProc
  CondRunProc,         // runProc
  CondParamProc,       // paramProc
  CondConfigProc,      // configProc
  CondStringProc,      // stringProc
  CondStartProc,       // startProc
  CondEndProc,         // endProc

  &condScriptObj,      // script_obj
  CondScriptStrProc    // fssProc
};

FilterDefinition *fd_conditional;

#pragma comment (exestr, "Conditional filter for VirtualDub. Copyright (C) 2001-2003 Dmitri Schamschurko")
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
const FilterFunctions* gmy_filterFuncs = 0;
int                    gmy_VIRTUALDUB_FILTERDEF_VERSION = VIRTUALDUB_FILTERDEF_VERSION;

extern "C" {

int myinit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat)
{
gmy_filterFuncs = ff;
gmy_VIRTUALDUB_FILTERDEF_VERSION = vdfd_ver;
randomize();

dout(__FUNC__ " fm: %p, ff: %p, ver: %d, comp: %d", fm, ff, vdfd_ver, vdfd_compat);

#pragma warn -8060 // pos. incorrect assignment
  if (!(fd_conditional = ff->addFilter(fm, &filterDef_conditional, sizeof(FilterDefinition))))
#pragma warn .8060
  {
    return 1;
  }

  vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
  vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

  installExternalPlugins();

  return 0;
}

void mydeinit(FilterModule *fm, const FilterFunctions *ff)
{
dout(__FUNC__);

  ff->removeFilter(fd_conditional);
  RemoveExternalPlugins();
}

//
// Borland C++ Builder(TM) prepends all __cdecl function with the underscore.
// If we disable this option in the project settings, than we cannot link
// the standard library :(
//
// The only way I see is to fake the compiler:
// declare the functions as __stdcall, but implement it as __cdecl
//
// Any better ideas?
//
// I've used here __emit__ instead of __asm to avoid second compile pass
//
#ifdef __BORLANDC__
#pragma warn -8070 // function should return a value
#endif

#ifdef __BORLANDC__
int __declspec(dllexport, naked) __stdcall VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
  __emit__(0x55);         // push ebp
  __emit__(0x8B, 0xEC);   // mov ebp, esp
#else
int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
#endif

  if (myinit2(fm, ff, vdfd_ver, vdfd_compat))
  {
#ifdef __BORLANDC__
    __emit__(0xBB, 1i32); // mox eax, 1
    __emit__(0x5D);       // pop ebp
    __emit__(0xC3);       // ret
#else
    return 1;
#endif
  }

#ifdef __BORLANDC__
  __emit__(0x33, 0xC0);   // xor eax, eax
  __emit__(0x5D);         // pop ebp
  __emit__(0xC3);         // ret
#else
  return 0;
#endif
}

#ifdef __BORLANDC__
void __declspec(dllexport, naked) __stdcall VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
  __emit__(0x55);         // push ebp
  __emit__(0x8B, 0xEC);   // mov ebp, esp
#else
void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
#endif

  mydeinit(fm, ff);

#ifdef __BORLANDC__
  __emit__(0x5D);         // pop ebp
  __emit__(0xC3);         // ret
#endif
}

#ifdef __BORLANDC__
#pragma warn .8070
#endif
}

