//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#ifndef f_ConditionDataH
#define f_ConditionDataH

#include <filter.h>
#include <condfun.h>

#define max3(a,b,c) max(a, max(b, c))
#define min3(a,b,c) min(a, min(b, c))

#if defined(_DEBUG) || defined(_XDEBUG)
#include <stdio.h>
void dout(const char* format, ...)
{
  char buffer[1024];
  buffer[1022] = 0;
  va_list val;
  va_start(val, format);
  _vsnprintf(buffer, 1022, format, val);
  va_end(val);
  char* end = strchr(buffer, 0);
  *end++ = '\n';
  *end   = 0;

  OutputDebugString(buffer);
}
#else
void inline dout(const char* format, ...)
{
}
#endif

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class VFBitmapInfo {
  public:
    int                     depth;
    PixCoord                w, h;
    PixOffset               pitch;
    PixOffset               modulo;
    PixOffset               size;
    PixOffset               offset;
    DWORD                   dwFlags;

  public:
    VFBitmapInfo(const VFBitmap& Bitmap);
    void Restore(VFBitmap& Bitmap) const;
    bool operator== (const VFBitmapInfo& o) const;
    bool operator!= (const VFBitmapInfo& o) const
    { return ! this->operator==(o); }
};

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class CCondFilterInstance {
  private:
    CCondFilterInstance& operator=(const CCondFilterInstance&);
    CCondFilterInstance(const CCondFilterInstance&);

  protected:
    int RefCounter;
    bool fNoDeinit;
    ~CCondFilterInstance();

  public:
    bool Started;

  public:
    int   iSize;
    char* pData;

    void IncRef();
    CCondFilterInstance(FilterActivation* fa, const FilterFunctions *ff);
    void DeleteInstance(FilterActivation* fa, const FilterFunctions *ff);
    void ForceNoDeinit()
    { fNoDeinit = true; }
    void __fastcall CallStartProc(FilterActivation* fa, const FilterFunctions *ff);
    void __fastcall CallEndProc(FilterActivation* fa, const FilterFunctions *ff);
};

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class CFormula;
class CExecutionContext;
struct ConditionalFilterData;
class CBranch {
  public:
    CBranch*               Next;
    CCondFilterInstance*   pInstance;
    bool                   IsElse;
    long                   NeedFlags;

  private:
    CBranch& operator=(CBranch&);

  protected:
    virtual ~CBranch();

  public:
    CBranch(FilterActivation* fa, const FilterFunctions *ff, bool VIsElse);
    CBranch(const CBranch& Other);
    bool __fastcall HasElse() const;
    void DeleteBranch(FilterActivation* fa, const FilterFunctions *ff);

    virtual CBranch* Clone() = 0;
    virtual CCondFilterInstance* __fastcall Evaluate(CExecutionContext& Context, int& Which) const = 0;

    void __fastcall CallStartProc(FilterActivation* fa, const FilterFunctions *ff);
    void __fastcall CallEndProc(FilterActivation* fa, const FilterFunctions *ff);
};

class CFormulaObject {
  private:
    CFormula   *pFormula;
    char       *pFormulaText;
    long       lRefCounter;

    __fastcall ~CFormulaObject();

  public:
    __fastcall CFormulaObject(CFormula* Formula, const char* FormulaText);
    void __fastcall IncRef();
    void __fastcall DeleteObject();
    bool __fastcall Evaluate(CExecutionContext& Context) const;
    const char* GetFormulaText() const;
    long  __fastcall GetNeedFlags() const;
};
class CIFBranch: public CBranch {
  public:
    CFormulaObject* pFormula;

  protected:
    virtual ~CIFBranch();

  public:
    CIFBranch(FilterActivation* fa, const FilterFunctions *ff, CFormula* Formula, const char* FormulaText);
    CIFBranch(const CIFBranch& Other);
    virtual CBranch* Clone();
    virtual CCondFilterInstance* __fastcall Evaluate(CExecutionContext& Context, int& Which) const;
};

class CELSEBranch: public CBranch {
  public:
    CELSEBranch(FilterActivation* fa, const FilterFunctions *ff);
    CELSEBranch(const CELSEBranch& Other);
    virtual CBranch* Clone();
    virtual CCondFilterInstance* __fastcall Evaluate(CExecutionContext& Context, int& Which) const;
};

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class CNumeric;
struct ConditionalFilterData {
  FilterDefinition*      Filter;
  const FilterFunctions* ff;

  FilterDefinition* OwnFilter;
  void*             OwnFilterData;

  bool              Disabled;
  const char*       DisabledReason;
  bool              HaveToSwap;

  CBranch*          Branches;
  CBranch*          PreviewBranch;
  CNumeric**        MonitorPreview;
  HANDLE            MonitorPreviewHandle;

  ConditionalFilterData();
};
typedef struct ConditionalFilterData CFD;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
struct ExtendedExternalInfo: public ExternalFunctionInfo {
  HINSTANCE hInstance;
  ExtendedExternalInfo& operator=(ExternalFunctionInfo& Other);
};
extern ExtendedExternalInfo ExternalFunctions[100];
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#endif

