//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "f_ConditionFilterList.h"
#include "f_ConditionData.h"
#include "f_VD15xProxy.h"

#include <filter.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TFilterListDlg *FilterListDlg;
//---------------------------------------------------------------------------
__fastcall TFilterListDlg::TFilterListDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TFilterListDlg::FilterListClick(TObject *Sender)
{
  FilterList->Selected = FilterList->ItemFocused;
}
//---------------------------------------------------------------------------
int ColumnToSort = 0;
void __fastcall TFilterListDlg::FilterListColumnClick(TObject *Sender,
      TListColumn *Column)
{
  ColumnToSort = Column->Index;
  static_cast<TCustomListView*>(Sender)->AlphaSort();
}
//---------------------------------------------------------------------------
void __fastcall TFilterListDlg::FilterListCompare(TObject *Sender,
      TListItem *Item1, TListItem *Item2, int Data, int &Compare)
{
  if (ColumnToSort == 0)
    Compare = CompareText(Item1->Caption,Item2->Caption);
  else
  {
    int ix = ColumnToSort - 1;
    Compare = CompareText(Item1->SubItems->Strings[ix], Item2->SubItems->Strings[ix]);
  }
}
//---------------------------------------------------------------------------
void __fastcall TFilterListDlg::FilterListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
  if (Selected && Item) {
//    FilterDefinition* Filter = reinterpret_cast<FilterDefinition*>(Item->Data);
//    if (Filter->desc)
//      Description->Caption = Filter->desc;
//    else
//      Description->Caption = "";
    Description->Caption = Item->SubItems->Strings[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TFilterListDlg::FormActivate(TObject *Sender)
{
  FilterList->Items->Clear();

  CFilterDefinitionHelper it(fd_conditional);

  FilterDefinition* Filter = it.GetNext(fd_conditional);
//  FilterDefinition* Filter = fd_conditional->next;
  while (Filter && Filter != fd_conditional)
  {
    CFilterInfo fi = it.GetInfo(Filter);

    TListItem *Item = FilterList->Items->Add();
    Item->Caption = fi.Name;
    Item->SubItems->Add(fi.Maker);
    Item->SubItems->Add(fi.HasConfig ? "yes" : "no");
    Item->SubItems->Add(fi.Desc);

    Item->Data = Filter;

    Filter = it.GetNext(Filter);
//    Filter = Filter->next;
  }

  Filter = it.GetPrev(fd_conditional);
//  Filter = fd_conditional->prev;
  while (Filter && Filter != fd_conditional)
  {
    CFilterInfo fi = it.GetInfo(Filter);

    TListItem *Item = FilterList->Items->Add();
    Item->Caption = fi.Name;
    Item->SubItems->Add(fi.Maker);
    Item->SubItems->Add(fi.HasConfig ? "yes" : "no");
    Item->SubItems->Add(fi.Desc);

    Item->Data = Filter;

    Filter = it.GetPrev(Filter);
  }

  if (!FilterList->Items->Count)
    OkButton->Enabled = false;
  else {
    FilterList->Selected = FilterList->Items->Item[0];
    FilterList->Selected->Focused = true;
  }

#ifdef ENABLE_RESIZING
  if (BorderStyle == bsDialog)
    BorderStyle = bsSizeable;
#endif
}
//---------------------------------------------------------------------------
void __fastcall TFilterListDlg::FilterListDblClick(TObject *Sender)
{
  if (FilterList->Selected)
    ModalResult = mrOk;
}
//---------------------------------------------------------------------------

