%{

//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#ifdef __BCPLUSPLUS__
#pragma warn -ccc
#pragma warn -sig
#endif

class CNumeric;
class CFormula;
class NFunction;
#include "parser.tab.hpp"

/*
 * Wir versuchen es mit flex Optionen
**

//
// sonst wird versucht yywrap als extern "C" zu deklarieren
//
#define YY_SKIP_YYWRAP

//
// sonst kriegen wir probleme mit isatty
//
#define YY_NEVER_INTERACTIVE 1

***
 */

// #define YY_USER_ACTION printf("** %s **\n", yytext);
int comment_depth = 0;

%}

%option caseless never-interactive noyywrap outfile="lexer.cpp"

NUMBER        [0-9]+
ID            [a-z][a-z0-9_]*

%x COMMENT

%%

<INITIAL>"/*" comment_depth = 1; BEGIN(COMMENT);
<COMMENT>"*/" if (!--comment_depth) BEGIN(INITIAL);
<COMMENT>[^*]+
<COMMENT>"*"+[^/]
<COMMENT><<EOF>>       BEGIN(INITIAL); return -1;
"+"           return PLUS;
"-"           return MINUS;
"*"           return TIMES;
"/"           return DIV;
"%"           return DIVREST;
"("           return LP;
")"           return RP;
"<"           return LESS;
">"           return GREATER;
"<="          return LESSEQ;
">="          return GREATEREQ;
","           return COMMA;
"!="          return NOTEQ;
"<>"          return NOTEQ;
"="           return EQUAL;
"and"         return AND;
"or"          return OR;
"not"         return NOT;
"true"        return T_TRUE;
"false"       return T_FALSE;
"frame"       return FRAME;
"stamp"       return STAMP;
"gray"        return GRAY;
"grey"        return GRAY;
"luma"        return LUMA;
"luma601"     return LUMA601;
"luma709"     return LUMA709;
"rand"("dom")?    return RAND;
"bright"("ness")? return BRIGHTNESS;
"light"("ness")?  return LIGHTNESS;
"sat"("uration")?        return SATURATION;
"sat"("uration")?"HSB"   return SATURATIONHSB;
"sat"("uration")?"HLS"   return SATURATIONHLS;
"min"         return T_MIN;
"max"         return T_MAX;
"avg"         return T_AVG;
{NUMBER}      yylval.val = atoi( yytext ); return NUMBER;
{ID}          yylval.id = yytext; return T_ID;
[ \t\n\r]
<INITIAL><<EOF>>       /*yyterminate();*/ return MYEOS;
.             BEGIN(INITIAL); return -1;
%%

#ifdef __BCPLUSPLUS__
#pragma warn .ccc
#pragma warn .sig
#endif
