//---------------------------------------------------------------------------
//      The original file was modified for the following application:
//
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 2.0     March 2003
//---------------------------------------------------------------------------
//
// All inline methods with asm instructions are replaced
// by conventional call with implementation in atomicint.cpp
//
//---------------------------------------------------------------------------
#ifndef f_VD2_SYSTEM_ATOMIC_H
#define f_VD2_SYSTEM_ATOMIC_H

#pragma warning(push)
#pragma warning(disable: 4035)

class VDAtomicInt {
protected:
	volatile int n;

public:
	VDAtomicInt() {}
	VDAtomicInt(int v) : n(v) {}

	bool operator!() const { return !n; }
	bool operator!=(int v) const { return n!=v; }
	bool operator==(int v) const { return n==v; }
	bool operator<=(int v) const { return n<=v; }
	bool operator>=(int v) const { return n>=v; }
	bool operator<(int v) const { return n<v; }
	bool operator>(int v) const { return n>v; }

	///////////////////////////////

	static  int staticExchange(volatile int *dst, int v);
	static  void staticIncrement(volatile int *dst);
	static  void staticDecrement(volatile int *dst);
	static  bool staticDecrementTestZero(volatile int *dst);
	static  void staticAdd(volatile int *dst, int v);
	static  int staticAddResult(volatile int *dst, int v);

	///////////////////////////////

	int operator=(int v) { return n = v; }

	void operator++();
	void operator--();

	void operator++(int) { ++*this; }
	void operator--(int) { --*this; }

	void operator+=(int v);
	void operator-=(int v);

	operator int() const {
		return n;
	}

	int xchg(int v);

	bool inctestzero();
	bool dectestzero();

	bool addcarry(int v);
	bool subcarry(int v);

	// 486 only, but much nicer.  They return the actual result.

	int inc();
	int dec();

	int add(int v);

	// These return the result before the operation, which is more  with
	// what XADD allows us to do.

	int postinc();
	int postdec();

	int postadd(int v);

};

#pragma warning(pop)
#endif
